/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2009 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Author: David Zeuthen <davidz@redhat.com>
 */

#ifndef __GDU_CONNECT_TO_SERVER_DIALOG_H
#define __GDU_CONNECT_TO_SERVER_DIALOG_H

#include <gdu-gtk/gdu-gtk.h>

G_BEGIN_DECLS

#define GDU_TYPE_CONNECT_TO_SERVER_DIALOG         (gdu_connect_to_server_dialog_get_type())
#define GDU_CONNECT_TO_SERVER_DIALOG(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GDU_TYPE_CONNECT_TO_SERVER_DIALOG, GduConnectToServerDialog))
#define GDU_CONNECT_TO_SERVER_DIALOG_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), GDU_TYPE_CONNECT_TO_SERVER_DIALOG, GduConnectToServerDialogClass))
#define GDU_IS_CONNECT_TO_SERVER_DIALOG(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDU_TYPE_CONNECT_TO_SERVER_DIALOG))
#define GDU_IS_CONNECT_TO_SERVER_DIALOG_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GDU_TYPE_CONNECT_TO_SERVER_DIALOG))
#define GDU_CONNECT_TO_SERVER_DIALOG_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GDU_TYPE_CONNECT_TO_SERVER_DIALOG, GduConnectToServerDialogClass))

typedef struct GduConnectToServerDialogClass   GduConnectToServerDialogClass;
typedef struct GduConnectToServerDialogPrivate GduConnectToServerDialogPrivate;

struct GduConnectToServerDialog
{
        GtkDialog parent;

        /*< private >*/
        GduConnectToServerDialogPrivate *priv;
};

struct GduConnectToServerDialogClass
{
        GtkDialogClass parent_class;
};

GType       gdu_connect_to_server_dialog_get_type      (void) G_GNUC_CONST;
GtkWidget*  gdu_connect_to_server_dialog_new           (GtkWindow                 *parent);
gchar      *gdu_connect_to_server_dialog_get_user_name (GduConnectToServerDialog  *dialog);
gchar      *gdu_connect_to_server_dialog_get_address   (GduConnectToServerDialog  *dialog);

G_END_DECLS

#endif  /* __GDU_CONNECT_TO_SERVER_DIALOG_H */

